##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""'browser' namespace directive tests
"""

import os
import sys
import unittest
from io import StringIO

import zope.security.management
from zope.configuration.xmlconfig import XMLConfig
from zope.configuration.xmlconfig import xmlconfig
from zope.interface import Interface
from zope.interface import directlyProvides
from zope.interface import implementer
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.testing import cleanup
from zope.traversing.adapters import DefaultTraversable
from zope.traversing.interfaces import ITraversable

import zope.browsermenu
from zope import component
from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IMenuItemType
from zope.browsermenu.menu import BrowserMenu
from zope.browsermenu.menu import getFirstMenuItem


class IV(Interface):

    def index():
        """Return index text"""


class IC(Interface):
    """An interface"""


@implementer(IV)
class V1:
    pass


tests_path = os.path.join(
    os.path.dirname(zope.browsermenu.__file__),
    'tests')


template = """<configure
   xmlns='http://namespaces.zope.org/zope'
   xmlns:browser='http://namespaces.zope.org/browser'
   i18n_domain='zope'>
   %s
   </configure>"""


request = TestRequest()


class M1(BrowserMenu):
    pass


class V2(V1):
    pass


class VT(V1):
    pass


@implementer(IC)
class Ob:
    pass


ob = Ob()


class NCV:
    """non callable view"""


class CV(NCV):
    """callable view"""

    def __call__(self):
        raise AssertionError("Not called")


@implementer(Interface)
class C_w_implements(NCV):

    def index(self):
        raise AssertionError("Not called")


class ITestMenu(Interface):
    """Test menu."""


directlyProvides(ITestMenu, IMenuItemType)


class ITestLayer(IBrowserRequest):
    """Test Layer."""


class ITestSkin(ITestLayer):
    """Test Skin."""


class MyResource:
    pass


class Test(cleanup.CleanUp, unittest.TestCase):

    def setUp(self):
        super().setUp()
        XMLConfig('meta.zcml', zope.browsermenu)()
        component.provideAdapter(DefaultTraversable, (None,), ITraversable)

    def tearDown(self):
        if 'test_menu' in dir(sys.modules['zope.app.menus']):
            delattr(sys.modules['zope.app.menus'], 'test_menu')
        super().tearDown()

    def testMenuOverride(self):
        self.assertEqual(
            component.queryMultiAdapter((ob, request), name='test'),
            None)

        xmlconfig(StringIO(template % (
            '''
            <browser:menu
                id="test_menu" title="Test menu" />
            <browser:menuItem
                action="@@test"
                for="zope.component.testfiles.views.IC"
                permission="zope.Public"
                menu="test_menu"
                title="Test View"
                />
            '''
        )))
        menu1 = component.getUtility(IBrowserMenu, 'test_menu')
        menuItem1 = getFirstMenuItem('test_menu', ob, TestRequest())
        xmlconfig(StringIO(template % (
            '''
            <browser:menu
                id="test_menu" title="Test menu"
                class="zope.browsermenu.tests.test_directives.M1" />
            '''
        )))
        menu2 = component.getUtility(IBrowserMenu, 'test_menu')
        menuItem2 = getFirstMenuItem('test_menu', ob, TestRequest())
        self.assertNotEqual(menu1, menu2)
        self.assertEqual(menuItem1, menuItem2)

    def testMenuItemNeedsFor(self):
        # <browser:menuItem> directive fails if no 'for' argument was provided
        from zope.configuration.exceptions import ConfigurationError
        self.assertRaises(
            ConfigurationError,
            xmlconfig,
            StringIO(
                template %
                '''
                <browser:menu
                    id="test_menu" title="Test menu" />
                <browser:menuItem
                    title="Test Entry"
                    menu="test_menu"
                    action="@@test"
                />
                '''
            ))

        # it works, when the argument is there and a valid interface
        xmlconfig(StringIO(
            template %
            '''
            <browser:menuItem
                for="zope.component.testfiles.views.IC"
                title="Test Entry"
                menu="test_menu"
                action="@@test"
            />
            '''
        ))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test fields.
"""
import doctest
import unittest

from zope.testing import cleanup


class TestMenuField(unittest.TestCase):

    def test_unconfigured(self):
        from zope.configuration.exceptions import ConfigurationError
        from zope.schema import ValidationError

        from zope.browsermenu.field import MenuField

        class Resolver:
            def resolve(self, name):
                raise ConfigurationError(name)

        field = MenuField()
        field = field.bind(Resolver())

        with self.assertRaises(ValidationError):
            field.fromUnicode('')


def test_suite():
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocTestSuite(
            'zope.browsermenu.field',
            setUp=lambda test: cleanup.setUp(),
            tearDown=lambda test: cleanup.tearDown()),
    ))



#############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Menu field
"""
from zope.component import queryUtility
from zope.configuration.exceptions import ConfigurationError
from zope.configuration.fields import GlobalObject
from zope.schema import ValidationError

from zope.browsermenu.interfaces import IMenuItemType


class MenuField(GlobalObject):
    r"""This fields represents a menu (item type).

    Besides being able to look up the menu by importing it, we also try
    to look up the name in the site manager.

    >>> from zope.interface import directlyProvides
    >>> from zope.interface.interface import InterfaceClass

    >>> menu1 = InterfaceClass('menu1', (),
    ...                        __doc__='Menu Item Type: menu1',
    ...                        __module__='zope.app.menus')
    >>> directlyProvides(menu1, IMenuItemType)

    >>> menus = None
    >>> class Resolver(object):
    ...     def resolve(self, path):
    ...         if path.startswith('zope.app.menus') and \
    ...             hasattr(menus, 'menu1') or \
    ...             path == 'zope.browsermenu.menus.menu1':
    ...             return menu1
    ...         raise ConfigurationError('menu1')

    >>> field = MenuField()
    >>> field = field.bind(Resolver())

    Test 1: Import the menu
    -----------------------

    >>> field.fromUnicode('zope.browsermenu.menus.menu1') is menu1
    True

    Test 2: We have a shortcut name. Import the menu from `zope.app.menus1`.
    ------------------------------------------------------------------------

    >>> from types import ModuleType as module
    >>> import sys
    >>> menus = module('menus')
    >>> old = sys.modules.get('zope.app.menus', None)
    >>> sys.modules['zope.app.menus'] = menus
    >>> setattr(menus, 'menu1', menu1)

    >>> field.fromUnicode('menu1') is menu1
    True

    >>> if old is not None:
    ...     sys.modules['zope.app.menus'] = old

    Test 3: Get the menu from the Site Manager
    ------------------------------------------

    >>> from zope.component import provideUtility
    >>> provideUtility(menu1, IMenuItemType, 'menu1')

    >>> field.fromUnicode('menu1') is menu1
    True
    """

    def fromUnicode(self, u):
        name = str(u.strip())

        # queryUtility can never raise ComponentLookupError
        value = queryUtility(IMenuItemType, name)
        if value is not None:
            self.validate(value)
            return value

        try:
            value = self.context.resolve('zope.app.menus.' + name)
        except ConfigurationError:
            try:
                value = self.context.resolve(name)
            except ConfigurationError as v:
                raise ValidationError(v)

        self.validate(value)
        return value



#############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Menu ZCML directives
"""
from zope.configuration.fields import GlobalInterface
from zope.configuration.fields import GlobalObject
from zope.configuration.fields import MessageID
from zope.interface import Interface
from zope.schema import Id
from zope.schema import Int
from zope.schema import TextLine
from zope.security.zcml import Permission

from zope.browsermenu.field import MenuField


class IMenuDirective(Interface):
    """Define a browser menu"""

    id = TextLine(
        title="The name of the menu.",
        description="This is, effectively, an id.",
        required=False
    )

    title = MessageID(
        title="Title",
        description="A descriptive title for documentation purposes",
        required=False
    )

    description = MessageID(
        title="Description",
        description="A description title of the menu.",
        required=False
    )

    class_ = GlobalObject(
        title="Menu Class",
        description="The menu class used to generate the menu.",
        required=False
    )

    interface = GlobalInterface(
        title="The menu's interface.",
        required=False
    )


class IMenuItemsDirective(Interface):
    """
    Define a group of browser menu items

    This directive is useful when many menu items are defined for the
    same interface and menu.
    """

    menu = MenuField(
        title="Menu name",
        description="The (name of the) menu the items are defined for",
        required=True,
    )

    for_ = GlobalObject(
        title="Interface",
        description="The interface the menu items are defined for",
        required=True
    )

    layer = GlobalInterface(
        title="Layer",
        description="The Layer for which the item is declared.",
        required=False
    )

    permission = Permission(
        title="The permission needed access the item",
        description="""
        This can usually be inferred by the system, however, doing so
        may be expensive. When displaying a menu, the system tries to
        traverse to the URLs given in each action to determine whether
        the url is accessible to the current user. This can be
        avoided if the permission is given explicitly.""",
        required=False
    )


class IMenuItem(Interface):
    """Common menu item configuration
    """

    title = MessageID(
        title="Title",
        description="The text to be displayed for the menu item",
        required=True
    )

    description = MessageID(
        title="A longer explanation of the menu item",
        description="""
        A UI may display this with the item or display it when the
        user requests more assistance.""",
        required=False
    )

    icon = TextLine(
        title="Icon Path",
        description="Path to the icon resource representing this menu item.",
        required=False
    )

    permission = Permission(
        title="The permission needed access the item",
        description="""
        This can usually be inferred by the system, however, doing so
        may be expensive. When displaying a menu, the system tries to
        traverse to the URLs given in each action to determine whether
        the url is accessible to the current user. This can be
        avoided if the permission is given explicitly.""",
        required=False
    )

    filter = TextLine(
        title="A condition for displaying the menu item",
        description="""
        The condition is given as a TALES expression. The expression
        has access to the variables:

        context -- The object the menu is being displayed for

        request -- The browser request

        nothing -- None

        The menu item will not be displayed if there is a filter and
        the filter evaluates to a false value.""",
        required=False
    )

    order = Int(
        title="Order",
        description="A relative position of the menu item in the menu.",
        required=False,
        default=0
    )

    item_class = GlobalObject(
        title="Menu item class",
        description="""
        A class to be used as a factory for creating menu item""",
        required=False
    )


class IMenuItemSubdirective(IMenuItem):
    """Define a menu item within a group of menu items"""

    action = TextLine(
        title="The relative url to use if the item is selected",
        description="""
        The url is relative to the object the menu is being displayed
        for.""",
        required=True
    )


class IMenuItemDirective(IMenuItemsDirective, IMenuItemSubdirective):
    """Define one menu item"""


class ISubMenuItemSubdirective(IMenuItem):
    """Define a menu item that represents a a sub menu.

    For a sub-menu menu item, the action is optional, this the item itself
    might not represent a destination, but just an entry point to the sub menu.
    """

    action = TextLine(
        title="The relative url to use if the item is selected",
        description="""
        The url is relative to the object the menu is being displayed
        for.""",
        required=False
    )

    submenu = TextLine(
        title="Sub-Menu Id",
        description="The menu that will be used to provide the sub-entries.",
        required=True,
    )


class ISubMenuItemDirective(IMenuItemsDirective, ISubMenuItemSubdirective):
    """Define one menu item"""


class IAddMenuItemDirective(IMenuItem):
    """Define an add-menu item"""

    for_ = GlobalInterface(
        title="Interface",
        description="The interface the menu items are defined for",
        required=False
    )

    class_ = GlobalObject(
        title="Class",
        description="""
        A class to be used as a factory for creating new objects""",
        required=False
    )

    factory = Id(
        title="Factory",
        description="A factory id for creating new objects",
        required=False,
    )

    view = TextLine(
        title="Custom view name",
        description="The name of a custom add view",
        required=False,
    )

    menu = MenuField(
        title="Menu name",
        description="The (name of the) menu the items are defined for",
        required=False,
    )

    layer = GlobalInterface(
        title="The layer the custom view is declared for",
        description="The default layer for which the custom view is "
                    "applicable. By default it is applied to all layers.",
        required=False
    )



#############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test the addMenuItem directive

>>> from zope.browsermenu.metaconfigure import addMenuItem
>>> context = Context()
>>> addMenuItem(context, class_=X, title="Add an X",
...             permission="zope.ManageContent")
>>> context
[('utility',
  <InterfaceClass zope.component.interfaces.IFactory>,
  'BrowserAdd__zope.browsermenu.tests.test_addMenuItem.X'),
 (<function provideInterface>,
  <InterfaceClass zope.component.interfaces.IFactory>),
 ('adapter',
  (<InterfaceClass zope.browser.interfaces.IAdding>,
   <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
  <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
  'Add an X'),
 (<function provideInterface>,
  <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
 (<function provideInterface>,
  <InterfaceClass zope.browser.interfaces.IAdding>),
 (<function provideInterface>,
  <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
"""

import io
import pprint
import re
import unittest
from doctest import DocTestSuite

import zope.component
from zope.component.interface import provideInterface
from zope.configuration.xmlconfig import XMLConfig
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.testing import cleanup

import zope.browsermenu
from zope.browsermenu.metaconfigure import _checkViewFor


atre = re.compile(' at [0-9a-fA-Fx]+')


class IX(Interface):
    pass


class X:
    pass


class ILayerStub(IBrowserRequest):
    pass


class MenuStub:
    pass


class Context:
    info = ''

    def __init__(self):
        self.actions = []

    def action(self, discriminator, callable, args=(), kw=None, order=0):
        if discriminator is None:
            if callable is provideInterface:
                self.actions.append((callable, args[1]))  # name is args[0]
            elif callable is _checkViewFor:
                self.actions.append((callable, args[2]))
        else:
            self.actions.append(discriminator)

    def __repr__(self):
        stream = io.BytesIO() if str is bytes else io.StringIO()
        pprinter = pprint.PrettyPrinter(stream=stream, width=60)
        pprinter.pprint(self.actions)
        r = stream.getvalue()
        return (''.join(atre.split(r))).strip()


def test_w_factory():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem
    >>> context = Context()
    >>> addMenuItem(context, factory="x.y.z", title="Add an X",
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo")
    >>> context
    [('adapter',
      (<InterfaceClass zope.browser.interfaces.IAdding>,
       <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
      'Add an X'),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
     (<function provideInterface>,
      <InterfaceClass zope.browser.interfaces.IAdding>),
     (<function provideInterface>,
      <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
    """  # noqa: E501 (long line)


def test_w_factory_and_view():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem

    >>> context = Context()
    >>> addMenuItem(context, factory="x.y.z", title="Add an X",
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo", view="AddX")
    >>> context
    [(<function _checkViewFor>, 'AddX'),
     ('adapter',
      (<InterfaceClass zope.browser.interfaces.IAdding>,
       <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
      'Add an X'),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
     (<function provideInterface>,
      <InterfaceClass zope.browser.interfaces.IAdding>),
     (<function provideInterface>,
      <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
    """  # noqa: E501 (long line)


def test_w_factory_class_view():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem

    >>> context = Context()
    >>> addMenuItem(context, class_=X, title="Add an X",
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo", view="AddX")
    >>> context
    [('utility',
      <InterfaceClass zope.component.interfaces.IFactory>,
      'BrowserAdd__zope.browsermenu.tests.test_addMenuItem.X'),
     (<function provideInterface>,
      <InterfaceClass zope.component.interfaces.IFactory>),
     (<function _checkViewFor>, 'AddX'),
     ('adapter',
      (<InterfaceClass zope.browser.interfaces.IAdding>,
       <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
      'Add an X'),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
     (<function provideInterface>,
      <InterfaceClass zope.browser.interfaces.IAdding>),
     (<function provideInterface>,
      <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
    """  # noqa: E501 (long line)


def test_w_for_factory():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem

    >>> context = Context()
    >>> addMenuItem(context, for_=IX, factory="x.y.z", title="Add an X",
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo")
    >>> context
    [(<function provideInterface>,
      <InterfaceClass zope.browsermenu.tests.test_addMenuItem.IX>),
     ('adapter',
      (<InterfaceClass zope.browsermenu.tests.test_addMenuItem.IX>,
       <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
      'Add an X'),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.tests.test_addMenuItem.IX>),
     (<function provideInterface>,
      <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
    """  # noqa: E501 (long line)


def test_w_factory_layer():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem

    >>> context = Context()
    >>> addMenuItem(context, factory="x.y.z", title="Add an X",
    ...             layer=ILayerStub,
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo")
    >>> context
    [('adapter',
      (<InterfaceClass zope.browser.interfaces.IAdding>,
       <InterfaceClass zope.browsermenu.tests.test_addMenuItem.ILayerStub>),
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
      'Add an X'),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
     (<function provideInterface>,
      <InterfaceClass zope.browser.interfaces.IAdding>),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.tests.test_addMenuItem.ILayerStub>)]
    """


def test_w_for_menu_factory():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem

    >>> context = Context()
    >>> addMenuItem(context, for_=IX, menu=MenuStub,
    ...             factory="x.y.z", title="Add an X",
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo")
    >>> context
    [(<function provideInterface>,
      <InterfaceClass zope.browsermenu.tests.test_addMenuItem.IX>),
     ('adapter',
      (<InterfaceClass zope.browsermenu.tests.test_addMenuItem.IX>,
       <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
      <class 'zope.browsermenu.tests.test_addMenuItem.MenuStub'>,
      'Add an X'),
     (<function provideInterface>,
      <class 'zope.browsermenu.tests.test_addMenuItem.MenuStub'>),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.tests.test_addMenuItem.IX>),
     (<function provideInterface>,
      <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
    """  # noqa: E501 (long line)


def test_w_factory_icon_extra_order():
    """
    >>> from zope.browsermenu.metaconfigure import addMenuItem

    >>> context = Context()
    >>> addMenuItem(context, factory="x.y.z", title="Add an X",
    ...             permission="zope.ManageContent", description="blah blah",
    ...             filter="context/foo", icon=u'/@@/icon.png', extra='Extra',
    ...             order=99)
    >>> context
    [('adapter',
      (<InterfaceClass zope.browser.interfaces.IAdding>,
       <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>),
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>,
      'Add an X'),
     (<function provideInterface>,
      <InterfaceClass zope.browsermenu.interfaces.AddMenu>),
     (<function provideInterface>,
      <InterfaceClass zope.browser.interfaces.IAdding>),
     (<function provideInterface>,
      <InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>)]
    """  # noqa: E501 (long line)


class TestAddMenuItem(cleanup.CleanUp, unittest.TestCase):

    def setUp(self):
        super().setUp()
        XMLConfig('meta.zcml', zope.component)()
        XMLConfig('meta.zcml', zope.browsermenu)()

    def test_addMenuItemDirectives(self):
        XMLConfig('tests/addmenuitems.zcml', zope.browsermenu)()


def test_suite():
    return unittest.TestSuite((
        DocTestSuite(),
        unittest.defaultTestLoader.loadTestsFromName(__name__),
    ))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Menu implementation code
"""
import sys

from zope.component import getAdapters
from zope.component import getUtility
from zope.interface import Interface
from zope.interface import implementer
from zope.interface import providedBy
from zope.interface.interfaces import IInterface
from zope.pagetemplate.engine import Engine
from zope.publisher.browser import BrowserView
from zope.security import canAccess
from zope.security import checkPermission
from zope.security.interfaces import Forbidden
from zope.security.interfaces import Unauthorized
from zope.security.proxy import removeSecurityProxy
from zope.traversing.publicationtraverse import PublicationTraverser

from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IBrowserMenuItem
from zope.browsermenu.interfaces import IBrowserSubMenuItem
from zope.browsermenu.interfaces import IMenuAccessView
from zope.browsermenu.interfaces import IMenuItemType


@implementer(IBrowserMenu)
class BrowserMenu:
    """Browser Menu"""

    def __init__(self, id, title='', description=''):
        self.id = id
        self.title = title
        self.description = description

    def getMenuItemType(self):
        return getUtility(IMenuItemType, self.id)

    def getMenuItems(self, object, request):
        """Return menu item entries in a TAL-friendly form."""
        result = []
        for _name, item in getAdapters((object, request),
                                       self.getMenuItemType()):
            if item.available():
                result.append(item)

        # Now order the result. This is not as easy as it seems.
        #
        # (1) Look at the interfaces and put the more specific menu entries
        #     to the front.
        # (2) Sort unambigious entries by order and then by title.
        ifaces = list(providedBy(removeSecurityProxy(object)).__iro__)
        max_key = len(ifaces)

        def iface_index(item):
            iface = item._for or Interface
            if IInterface.providedBy(iface):
                return ifaces.index(iface)
            if isinstance(removeSecurityProxy(object), item._for):
                # directly specified for class, this goes first.
                return -1
            # no idea. This goes last.
            return max_key  # pragma: no cover
        result = [(iface_index(item), item.order, item.title, item)
                  for item in result]
        result.sort()

        result = [
            {'title': title,
             'description': item.description,
             'action': item.action,
             'selected': (item.selected() and 'selected') or '',
             'icon': item.icon,
             'extra': item.extra,
             'submenu': (IBrowserSubMenuItem.providedBy(item) and
                         getMenu(item.submenuId, object, request)) or None}
            for index, order, title, item in result]

        return result


@implementer(IBrowserMenuItem)
class BrowserMenuItem(BrowserView):
    """Browser Menu Item Class"""

    title = ''
    description = ''
    action = ''
    extra = None
    order = 0
    permission = None
    filter = None
    icon = None
    _for = Interface

    def available(self):
        # Make sure we have the permission needed to access the menu's action
        if self.permission is not None:
            # If we have an explicit permission, check that we
            # can access it.
            if not checkPermission(self.permission, self.context):
                return False

        elif self.action != '':
            # Otherwise, test access by attempting access
            path = self.action
            pos = self.action.find('?')
            if pos >= 0:
                path = self.action[:pos]

            traverser = PublicationTraverser()
            try:
                view = traverser.traverseRelativeURL(
                    self.request, self.context, path)
            except (Unauthorized, Forbidden, LookupError):
                return False
            else:
                # we're assuming that view pages are callable
                # this is a pretty sound assumption
                if not canAccess(view, '__call__'):
                    return False  # pragma: no cover

        # Make sure that we really want to see this menu item
        if self.filter is not None:

            try:
                include = self.filter(Engine.getContext(
                    context=self.context,
                    nothing=None,
                    request=self.request,
                    modules=sys.modules,
                ))
            except Unauthorized:
                return False
            else:
                if not include:
                    return False

        return True

    def selected(self):
        request_url = self.request.getURL()

        normalized_action = self.action
        if self.action.startswith('@@'):
            normalized_action = self.action[2:]

        if request_url.endswith('/' + normalized_action):
            return True
        if request_url.endswith('/++view++' + normalized_action):
            return True
        if request_url.endswith('/@@' + normalized_action):
            return True

        return False


@implementer(IBrowserSubMenuItem)
class BrowserSubMenuItem(BrowserMenuItem):
    """Browser Menu Item Base Class"""

    submenuId = None

    def selected(self):
        if self.action == '':
            return False
        else:
            return super().selected()


def getMenu(id, object, request):
    """Return menu item entries in a TAL-friendly form."""
    menu = getUtility(IBrowserMenu, id)
    return menu.getMenuItems(object, request)


def getFirstMenuItem(id, object, request):
    """Get the first item of a menu."""
    items = getMenu(id, object, request)
    if items:
        return items[0]
    return None


@implementer(IMenuAccessView)
class MenuAccessView(BrowserView):
    """A view allowing easy access to menus."""

    def __getitem__(self, menuId):
        return getMenu(menuId, self.context, self.request)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Menu Directives Configuration Handlers
"""
import sys
# Create special modules that contain all menu item types
from types import ModuleType as module

from zope.browser.interfaces import IAdding
from zope.component import getGlobalSiteManager
from zope.component import queryUtility
from zope.component.interface import provideInterface
from zope.component.zcml import adapter
from zope.component.zcml import proxify
from zope.component.zcml import utility
from zope.configuration.exceptions import ConfigurationError
from zope.interface import Interface
from zope.interface.interface import InterfaceClass
from zope.pagetemplate.engine import Engine
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.security.checker import CheckerPublic
from zope.security.checker import InterfaceChecker
from zope.security.metaconfigure import ClassDirective

from zope.browsermenu.interfaces import AddMenu
from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IBrowserMenuItem
from zope.browsermenu.interfaces import IBrowserSubMenuItem
from zope.browsermenu.interfaces import IMenuItemType
from zope.browsermenu.menu import BrowserMenu
from zope.browsermenu.menu import BrowserMenuItem
from zope.browsermenu.menu import BrowserSubMenuItem


try:
    import zope.app  # noqa: F401 (unused symbol)
except ModuleNotFoundError:
    sys.modules['zope.app'] = module('app')
menus = module('menus')
sys.modules['zope.app.menus'] = menus


_order_counter = {}


def menuDirective(_context, id=None, class_=BrowserMenu, interface=None,
                  title='', description=''):
    """Register a new browser menu."""
    if id is None and interface is None:
        raise ConfigurationError(
            "You must specify the 'id' or 'interface' attribute.")

    if interface is None:
        if id in dir(menus):
            # reuse existing interfaces for the id, without this we are not
            # able to override menus.
            interface = getattr(menus, id)
        else:
            interface = InterfaceClass(id, (),
                                       __doc__='Menu Item Type: %s' % id,
                                       __module__='zope.app.menus')
            # Add the menu item type to the `menus` module.
            # Note: We have to do this immediately, so that directives using
            # the MenuField can find the menu item type.
            setattr(menus, id, interface)
        path = 'zope.app.menus.' + id
    else:
        path = interface.__module__ + '.' + interface.getName()

        # If an id was specified, make this menu available under this id.
        # Note that the menu will be still available under its path, since it
        # is an adapter, and the `MenuField` can resolve paths as well.
        if id is None:
            id = path
        else:
            # Make the interface available in the `zope.app.menus` module, so
            # that other directives can find the interface under the name
            # before the CA is setup.
            _context.action(
                discriminator=('browser', 'MenuItemType', path),
                callable=provideInterface,
                args=(path, interface, IMenuItemType, _context.info)
            )
            setattr(menus, id, interface)

    # Register the layer interface as an interface
    _context.action(
        discriminator=('interface', path),
        callable=provideInterface,
        args=(path, interface),
        kw={'info': _context.info}
    )

    # Register the menu item type interface as an IMenuItemType
    _context.action(
        discriminator=('browser', 'MenuItemType', id),
        callable=provideInterface,
        args=(id, interface, IMenuItemType, _context.info)
    )

    # Register the menu as a utility
    utility(_context, IBrowserMenu, class_(id, title, description), name=id)


def menuItemDirective(_context, menu, for_,
                      action, title, description='', icon=None, filter=None,
                      permission=None, layer=IDefaultBrowserLayer, extra=None,
                      order=0, item_class=None):
    """Register a single menu item."""
    return menuItemsDirective(_context, menu, for_, layer).menuItem(
        _context, action, title, description, icon, filter,
        permission, extra, order, item_class)


def subMenuItemDirective(_context, menu, for_, title, submenu,
                         action='', description='', icon=None, filter=None,
                         permission=None, layer=IDefaultBrowserLayer,
                         extra=None, order=0, item_class=None):
    """Register a single sub-menu menu item."""
    return menuItemsDirective(_context, menu, for_, layer).subMenuItem(
        _context, submenu, title, description, action, icon, filter,
        permission, extra, order, item_class)


class MenuItemFactory:
    """generic factory for menu items."""

    def __init__(self, factory, **kwargs):
        self.factory = factory
        if 'permission' in kwargs and kwargs['permission'] == 'zope.Public':
            kwargs['permission'] = CheckerPublic
        self.kwargs = kwargs

    def __call__(self, context, request):
        item = self.factory(context, request)

        for key, value in self.kwargs.items():
            setattr(item, key, value)

        if item.permission is not None:
            checker = InterfaceChecker(IBrowserMenuItem, item.permission)
            item = proxify(item, checker)

        return item


class menuItemsDirective:
    """Register several menu items for a particular menu."""

    menuItemClass = BrowserMenuItem
    subMenuItemClass = BrowserSubMenuItem

    def __init__(self, _context, menu, for_, layer=IDefaultBrowserLayer,
                 permission=None):
        self.for_ = for_
        self.menuItemType = menu
        self.layer = layer
        self.permission = permission

    def menuItem(self, _context, action, title, description='',
                 icon=None, filter=None, permission=None, extra=None,
                 order=0, item_class=None):

        if filter is not None:
            filter = Engine.compile(filter)

        if permission is None:
            permission = self.permission

        if order == 0:
            order = _order_counter.get(self.for_, 1)
            _order_counter[self.for_] = order + 1

        if item_class is None:
            item_class = self.menuItemClass

        if not IBrowserMenuItem.implementedBy(item_class):
            raise ValueError(
                "Item class (%s) must implement IBrowserMenuItem" % item_class)

        factory = MenuItemFactory(
            item_class,
            title=title, description=description, icon=icon, action=action,
            filter=filter, permission=permission, extra=extra, order=order,
            _for=self.for_)
        adapter(_context, (factory,), self.menuItemType,
                (self.for_, self.layer), name=title)

    def subMenuItem(self, _context, submenu, title, description='',
                    action='', icon=None, filter=None, permission=None,
                    extra=None, order=0, item_class=None):
        filter = Engine.compile(filter) if filter is not None else None

        if permission is None:
            permission = self.permission

        if order == 0:
            order = _order_counter.get(self.for_, 1)
            _order_counter[self.for_] = order + 1

        if item_class is None:
            item_class = self.subMenuItemClass

        if not IBrowserSubMenuItem.implementedBy(item_class):
            raise ValueError(
                "Item class (%s) must implement IBrowserSubMenuItem"
                % item_class)

        factory = MenuItemFactory(
            item_class,
            title=title, description=description, icon=icon, action=action,
            filter=filter, permission=permission, extra=extra, order=order,
            _for=self.for_, submenuId=submenu)
        adapter(_context, (factory,), self.menuItemType,
                (self.for_, self.layer), name=title)

    def __call__(self, _context):
        """See menuItem or subMenuItem."""


def _checkViewFor(for_=None, layer=None, view_name=None):
    """Check if there is a view of that name registered for IAdding
    and IBrowserRequest. If not raise a ConfigurationError

    It will raise a ConfigurationError if :
        o view=""
        o if view_name is not registred
    """
    if view_name is None:
        raise ConfigurationError(
            "Within a addMenuItem directive the view attribute"
            " is optional but can\'t be empty"
        )

    gsm = getGlobalSiteManager()
    if gsm.adapters.lookup((for_, layer),
                           Interface, view_name) is None:
        raise ConfigurationError(
            "view name %s not found " % view_name
        )


def addMenuItem(_context, title, description='', menu=None, for_=None,
                class_=None, factory=None, view=None, icon=None, filter=None,
                permission=None, layer=IDefaultBrowserLayer, extra=None,
                order=0, item_class=None):
    """Create an add menu item for a given class or factory

    As a convenience, a class can be provided, in which case, a
    factory is automatically defined based on the class.  In this
    case, the factory id is based on the class name.

    """
    if for_ is not None:
        _context.action(
            discriminator=None,
            callable=provideInterface,
            args=('', for_)
        )
        forname = 'For' + for_.getName()
    else:
        for_ = IAdding
        forname = ''

    if menu is not None:
        if isinstance(menu, str):
            menu_name = menu
            menu = queryUtility(IMenuItemType, menu)
            if menu is None:
                raise ValueError("Missing menu id '%s'" % menu_name)

    if class_ is None:
        if factory is None:
            raise ValueError("Must specify either class or factory")
    else:
        if factory is not None:
            raise ValueError("Can't specify both class and factory")
        if permission is None:
            raise ValueError(
                "A permission must be specified when a class is used")
        factory = "BrowserAdd{}__{}.{}".format(
            forname, class_.__module__, class_.__name__)
        ClassDirective(_context, class_).factory(_context, id=factory)

    extra = {'factory': factory}

    if view:
        action = view
        # This action will check if the view exists
        _context.action(
            discriminator=None,
            callable=_checkViewFor,
            args=(for_, layer, view),
            order=999999
        )
    else:
        action = factory

    if menu is None:
        menu = AddMenu

    return menuItemsDirective(_context, menu, for_, layer=layer).menuItem(
        _context, action, title, description, icon, filter,
        permission, extra, order, item_class)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser Menu Item Tests
"""
import doctest
import pprint
import unittest

from zope.component import ComponentLookupError
from zope.testing import cleanup

from zope.browsermenu import menu


class Request:

    def __init__(self, url):
        self._url = url

    def getURL(self):
        return self._url


class TestMenuAccessView(unittest.TestCase):

    def test_getitem(self):
        v = menu.MenuAccessView(None, None)
        with self.assertRaises(ComponentLookupError):
            v['id']


class TestBrowserMenuItem(unittest.TestCase):

    def test_selected_ends_with_action(self):
        req = Request('/action')
        item = menu.BrowserMenuItem(None, req)
        item.action = 'action'

        self.assertTrue(item.selected())

    def test_available_with_query(self):
        req = Request('/action?query')
        item = menu.BrowserMenuItem(None, req)
        item.action = 'action?query'
        self.assertFalse(item.available())

    def test_available_filter_unauthorized(self):
        from zope.security.interfaces import Unauthorized

        req = Request('/action')
        item = menu.BrowserMenuItem(None, req)

        filter_called = []

        def f(_):
            filter_called.append(True)
            raise Unauthorized()

        item.filter = f

        self.assertFalse(item.available())
        self.assertTrue(filter_called)


def test_suite():
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocFileSuite(
            '../README.txt',
            setUp=lambda test: cleanup.setUp(),
            tearDown=lambda test: cleanup.tearDown(),
            globs={'pprint': pprint.pprint},
            optionflags=doctest.NORMALIZE_WHITESPACE),
    ))



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Menu-specific interfaces
"""
from zope.i18nmessageid import ZopeMessageFactory as _
from zope.interface import Interface
from zope.interface import directlyProvides
from zope.interface.interfaces import IInterface
from zope.schema import URI
from zope.schema import Int
from zope.schema import Text
from zope.schema import TextLine


class IMenuItemType(IInterface):
    """Menu item type

    Menu item types are interfaces that define classes of
    menu items.
    """


class AddMenu(Interface):
    """Special menu for providing a list of addable objects."""


directlyProvides(AddMenu, IMenuItemType)


class IBrowserMenu(Interface):
    """Menu

    Menus are objects that can return a list of menu items they contain. How
    they generate this list is up to them. Commonly, however, they will look
    up adapters that provide the ``IBrowserMenuItem`` interface.
    """

    id = TextLine(
        title=_("Menu Id"),
        description=_("The id uniquely identifies this menu."),
        required=True
    )

    title = TextLine(
        title=_("Menu title"),
        description=_("The title provides the basic label for the menu."),
        required=False
    )

    description = Text(
        title=_("Menu description"),
        description=_("A description of the menu. This might be shown "
                      "on menu pages or in pop-up help for menus."),
        required=False
    )

    def getMenuItems(object, request):
        """Return a TAL-friendly list of menu items.

        The object (acts like the context) and request can be used to select
        the items that are available.
        """


class IBrowserMenuItem(Interface):
    """Menu type

    An interface that defines a menu.
    """

    title = TextLine(
        title=_("Menu item title"),
        description=_("The title provides the basic label for the menu item."),
        required=True
    )

    description = Text(
        title=_("Menu item description"),
        description=_("A description of the menu item. This might be shown "
                      "on menu pages or in pop-up help for menu items."),
        required=False
    )

    action = TextLine(
        title=_("The URL to display if the item is selected"),
        description=_("When a user selects a browser menu item, the URL"
                      "given in the action is displayed. The action is "
                      "usually given as a relative URL, relative to the "
                      "object the menu item is for."),
        required=True
    )

    order = Int(
        title=_("Menu item ordering hint"),
        description=_("This attribute provides a hint for menu item ordering."
                      "Menu items will generally be sorted by the `for_`"
                      "attribute and then by the order.")
    )

    filter_string = TextLine(
        title=_("A condition for displaying the menu item"),
        description=_("The condition is given as a TALES expression. The "
                      "expression has access to the variables:\n"
                      "\n"
                      "context -- The object the menu is being displayed "
                      "for\n"
                      "\n"
                      "request -- The browser request\n"
                      "\n"
                      "nothing -- None\n"
                      "\n"
                      "The menu item will not be displayed if there is a \n"
                      "filter and the filter evaluates to a false value."),
        required=False)

    icon = URI(
        title=_("Icon URI"),
        description=_("URI of the icon representing this menu item"))

    def available():
        """Test whether the menu item should be displayed

        A menu item might not be available for an object, for example
        due to security limitations or constraints.
        """


class IBrowserSubMenuItem(IBrowserMenuItem):
    """A menu item that points to a sub-menu."""

    submenuId = TextLine(
        title=_("Sub-Menu Id"),
        description=_("The menu id of the menu that describes the "
                      "sub-menu below this item."),
        required=True)

    action = TextLine(
        title=_("The URL to display if the item is selected"),
        description=_("When a user selects a browser menu item, the URL "
                      "given in the action is displayed. The action is "
                      "usually given as a relative URL, relative to the "
                      "object the menu item is for."),
        required=False
    )


class IMenuAccessView(Interface):
    """View that provides access to menus"""

    def __getitem__(menu_id):
        """Get menu information

        Return a sequence of dictionaries with labels and
        actions, where actions are relative URLs.
        """



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser Menu Directives Tests
"""
import unittest

import zope.component
import zope.security
from zope.configuration.xmlconfig import XMLConfig
from zope.interface import Interface
from zope.interface import implementer
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.security.interfaces import Forbidden
from zope.security.interfaces import Unauthorized
from zope.testing import cleanup

import zope.browsermenu
from zope.browsermenu.interfaces import IBrowserMenu


template = """<configure
   xmlns='http://namespaces.zope.org/zope'
   xmlns:browser='http://namespaces.zope.org/browser'
   i18n_domain='zope'>
   %s
   </configure>"""


class I1(Interface):
    pass


class I11(I1):
    pass


class I12(I1):
    pass


class I111(I11):
    pass


@implementer(I1)
class C1:
    pass


class I2(Interface):
    pass


@implementer(I2)
class C2:
    pass


@implementer(IBrowserPublisher, I111)
class TestObject:

    def f(self):
        raise AssertionError("Never called")

    def publishTraverse(self, request, name):
        if name[:1] == 'f':
            raise Forbidden(name)
        if name[:1] == 'u':
            raise Unauthorized(name)
        return self.f


class IMyLayer(Interface):
    pass


class IMySkin(IMyLayer, IDefaultBrowserLayer):
    pass


class TestPermissions(cleanup.CleanUp, unittest.TestCase):

    def setUp(self):
        super().setUp()
        XMLConfig('meta.zcml', zope.browsermenu)()
        XMLConfig('meta.zcml', zope.security)()

    def testMenuItemsPermission(self):
        XMLConfig('tests/menus-permissions.zcml', zope.browsermenu)()

        menu = zope.component.getUtility(IBrowserMenu, 'test_id')
        # This is a bit icky, but the menu hides too much stuff from us.
        items = zope.component.getAdapters((C2(), TestRequest()),
                                           menu.getMenuItemType())
        item = list(items)[0][1]
        self.assertEqual("zope.View", item.permission)


class TestZCML(cleanup.CleanUp, unittest.TestCase):

    def setUp(self):
        super().setUp()
        XMLConfig('meta.zcml', zope.browsermenu)()

    def testMenusAndMenuItems(self):
        XMLConfig('tests/menus.zcml', zope.browsermenu)()

        menu = zope.browsermenu.menu.getMenu(
            'test_id', TestObject(), TestRequest())

        def d(n):
            return {'action': "a%s" % n,
                    'title': "t%s" % n,
                    'description': '',
                    'selected': '',
                    'submenu': None,
                    'icon': None,
                    'extra': None}

        self.assertEqual(menu[:-1], [d(5), d(6), d(3), d(2), d(1)])
        self.assertEqual(
            menu[-1],
            {'submenu': [{'submenu': None,
                          'description': '',
                          'extra': None,
                          'selected': '',
                          'action': 'a10',
                          'title': 't10',
                          'icon': None}],
             'description': '',
             'extra': None,
             'selected': '',
             'action': '',
             'title': 's1',
             'icon': None})

        first = zope.browsermenu.menu.getFirstMenuItem(
            'test_id', TestObject(), TestRequest())

        self.assertEqual(first, d(5))

    def testMenuItemWithLayer(self):
        XMLConfig('tests/menus.zcml', zope.browsermenu)()

        menu = zope.browsermenu.menu.getMenu(
            'test_id', TestObject(), TestRequest())
        self.assertEqual(len(menu), 6)

        menu = zope.browsermenu.menu.getMenu(
            'test_id', TestObject(), TestRequest(skin=IMyLayer))
        self.assertEqual(len(menu), 2)

        menu = zope.browsermenu.menu.getMenu(
            'test_id', TestObject(), TestRequest(skin=IMySkin))
        self.assertEqual(len(menu), 8)


class TestFunctional(unittest.TestCase):

    def test_checkViewFor_no_name(self):
        from zope.configuration.exceptions import ConfigurationError

        from zope.browsermenu.metaconfigure import _checkViewFor

        with self.assertRaisesRegex(ConfigurationError,
                                    "can't be empty"):
            _checkViewFor()

    def test_checkViewFor_not_registered(self):
        from zope.configuration.exceptions import ConfigurationError

        from zope.browsermenu.metaconfigure import _checkViewFor

        with self.assertRaisesRegex(ConfigurationError,
                                    "view name"):
            _checkViewFor(Interface, Interface, view_name=__name__)

    def test_addMenuItem_menu_string_name(self):
        from zope.browsermenu.metaconfigure import addMenuItem
        with self.assertRaisesRegex(ValueError,
                                    "Missing menu id 'foobar'"):
            addMenuItem(None, "Title", menu='foobar')

    def test_addMenuItem_no_class_no_factory(self):
        from zope.browsermenu.metaconfigure import addMenuItem
        with self.assertRaisesRegex(ValueError,
                                    "Must specify either class or factory"):
            addMenuItem(None, "Title")

    def test_addMenuItem_class_and_factory(self):
        from zope.browsermenu.metaconfigure import addMenuItem
        with self.assertRaisesRegex(ValueError,
                                    "Can't specify both class and factory"):
            addMenuItem(None, "Title", class_=self, factory=self)

    def test_addMenuItem_class_no_permission(self):
        from zope.browsermenu.metaconfigure import addMenuItem
        with self.assertRaisesRegex(ValueError,
                                    "A permission must be specified"):
            addMenuItem(None, "Title", class_=self, permission=None)

    def test_subMenuItemDirective_type_error(self):
        from zope.browsermenu.metaconfigure import subMenuItemDirective
        with self.assertRaises(TypeError):
            subMenuItemDirective(None, None, Interface, 'title', 'submenu')



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""zope.browsermenu setup
"""
import os

from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


long_description = (read('README.rst') + '\n\n' + read('CHANGES.rst'))
TESTS_REQUIRE = [
    'zope.testing',
    'zope.testrunner >= 6.4',
]

setup(
    name='zope.browsermenu',
    version='6.0',
    url='http://github.com/zopefoundation/zope.browsermenu/',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.dev',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope :: 3',
    ],
    description='Browser menu implementation for Zope.',
    long_description=long_description,
    license='ZPL-2.1',
    python_requires='>=3.9',
    install_requires=[
        'setuptools',
        'zope.browser',
        'zope.component>=3.7',
        'zope.configuration',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.pagetemplate>=3.5',
        'zope.publisher >= 4.2.1',
        'zope.schema',
        'zope.security',
        'zope.traversing>3.7',
    ],
    extras_require={
        'test': TESTS_REQUIRE,
    },
    include_package_data=True,
    zip_safe=False,
)
